@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/audit.c
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1   2009/11/11
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -28,7 +28,7 @@ static char *ccs_print_bprm(struct linux_binprm *bprm,
 			    struct ccs_page_dump *dump)
 {
 	static const int ccs_buffer_len = 4096 * 2;
-	char *buffer = kzalloc(ccs_buffer_len, GFP_KERNEL);
+	char *buffer = kzalloc(ccs_buffer_len, CCS_GFP_FLAGS);
 	char *cp;
 	char *last_start;
 	int len;
@@ -160,12 +160,12 @@ static char *ccs_print_header(struct ccs_request_info *r)
 	struct ccs_obj_info *obj = r->obj;
 	const u32 ccs_flags = current->ccs_flags;
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
-	const pid_t gpid = (pid_t) sys_getpid();
+	const pid_t gpid = (pid_t) ccsecurity_exports.sys_getpid();
 #else
 	const pid_t gpid = task_pid_nr(current);
 #endif
 	static const int ccs_buffer_len = 4096;
-	char *buffer = kmalloc(ccs_buffer_len, GFP_KERNEL);
+	char *buffer = kmalloc(ccs_buffer_len, CCS_GFP_FLAGS);
 	int pos;
 	if (!buffer)
 		return NULL;
@@ -180,7 +180,8 @@ static char *ccs_print_header(struct ccs_request_info *r)
 				" egid=%u suid=%u sgid=%u fsuid=%u fsgid=%u"
 				" state[0]=%u state[1]=%u state[2]=%u"
 				" type%s=execute_handler }",
-				(pid_t) sys_getpid(), (pid_t) sys_getppid(),
+				(pid_t) ccsecurity_exports.sys_getpid(),
+				(pid_t) ccsecurity_exports.sys_getppid(),
 				current_uid(), current_gid(), current_euid(),
 				current_egid(), current_suid(), current_sgid(),
 				current_fsuid(), current_fsgid(),
@@ -298,7 +299,7 @@ char *ccs_init_audit_log(int *len, struct ccs_request_info *r)
 		symlink = r->obj->symlink_target->name;
 		*len += 18 + strlen(symlink);
 	}
-	buf = kzalloc(*len, GFP_KERNEL);
+	buf = kzalloc(*len, CCS_GFP_FLAGS);
 	if (!buf)
 		goto out;
 	pos = snprintf(buf, (*len) - 1, "%s", header);
@@ -372,26 +373,27 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 
 #else
 
-static DECLARE_WAIT_QUEUE_HEAD(ccs_grant_log_wait);
-static DECLARE_WAIT_QUEUE_HEAD(ccs_reject_log_wait);
+static wait_queue_head_t ccs_audit_log_wait[2] = {
+	__WAIT_QUEUE_HEAD_INITIALIZER(ccs_audit_log_wait[0]),
+	__WAIT_QUEUE_HEAD_INITIALIZER(ccs_audit_log_wait[1]),
+};
 
 static DEFINE_SPINLOCK(ccs_audit_log_lock);
 
 /* Structure for audit log. */
-struct ccs_log_entry {
+struct ccs_audit_log_entry {
 	struct list_head list;
 	char *log;
 	int size;
 };
 
-/* The list for "struct ccs_log_entry". */
-static LIST_HEAD(ccs_grant_log);
+/* The list for "struct ccs_audit_log_entry". */
+static struct list_head ccs_audit_log[2] = {
+	LIST_HEAD_INIT(ccs_audit_log[0]),
+	LIST_HEAD_INIT(ccs_audit_log[1]),
+};
 
-/* The list for "struct ccs_log_entry". */
-static LIST_HEAD(ccs_reject_log);
-
-static unsigned int ccs_grant_log_count;
-static unsigned int ccs_reject_log_count;
+static unsigned int ccs_audit_log_count[2];
 
 /**
  * ccs_write_audit_log - Write audit log.
@@ -410,21 +412,19 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 	int pos;
 	int len;
 	char *buf;
-	struct ccs_log_entry *new_entry;
+	struct ccs_audit_log_entry *new_entry;
 	bool quota_exceeded = false;
+	struct ccs_preference *pref;
 	if (!r->domain)
 		r->domain = ccs_current_domain();
-	if (is_granted) {
-		if (ccs_grant_log_count >= ccs_profile(r->domain->profile)->
-		    audit->audit_max_grant_log
-		    || !ccs_get_audit(r->profile, r->type, true))
-			goto out;
-	} else {
-		if (ccs_reject_log_count >= ccs_profile(r->domain->profile)->
-		    audit->audit_max_reject_log
-		    || !ccs_get_audit(r->profile, r->type, false))
-			goto out;
-	}
+	pref = ccs_profile(r->domain->profile)->audit;
+	if (is_granted)
+		len = pref->audit_max_grant_log;
+	else
+		len = pref->audit_max_reject_log;
+	if (ccs_audit_log_count[is_granted] >= len ||
+	    !ccs_get_audit(r->profile, r->type, is_granted))
+		goto out;
 	va_start(args, fmt);
 	len = vsnprintf((char *) &pos, sizeof(pos) - 1, fmt, args) + 32;
 	va_end(args);
@@ -435,7 +435,7 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 	va_start(args, fmt);
 	vsnprintf(buf + pos, len - pos - 1, fmt, args);
 	va_end(args);
-	new_entry = kzalloc(sizeof(*new_entry), GFP_KERNEL);
+	new_entry = kzalloc(sizeof(*new_entry), CCS_GFP_FLAGS);
 	if (!new_entry) {
 		kfree(buf);
 		goto out;
@@ -452,13 +452,8 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 		quota_exceeded = true;
 	} else {
 		ccs_audit_log_memory_size += new_entry->size;
-		if (is_granted) {
-			list_add_tail(&new_entry->list, &ccs_grant_log);
-			ccs_grant_log_count++;
-		} else {
-			list_add_tail(&new_entry->list, &ccs_reject_log);
-			ccs_reject_log_count++;
-		}
+		list_add_tail(&new_entry->list, &ccs_audit_log[is_granted]);
+		ccs_audit_log_count[is_granted]++;
 	}
 	spin_unlock(&ccs_audit_log_lock);
 	if (quota_exceeded) {
@@ -466,10 +461,7 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 		kfree(new_entry);
 		goto out;
 	}
-	if (is_granted)
-		wake_up(&ccs_grant_log_wait);
-	else
-		wake_up(&ccs_reject_log_wait);
+	wake_up(&ccs_audit_log_wait[is_granted]);
 	error = 0;
  out:
 	ccs_update_task_state(r);
@@ -477,13 +469,14 @@ int ccs_write_audit_log(const bool is_granted, struct ccs_request_info *r,
 }
 
 /**
- * ccs_read_grant_log - Read a grant log.
+ * ccs_read_audit_log - Read an audit log.
  *
  * @head: Pointer to "struct ccs_io_buffer".
  */
-void ccs_read_grant_log(struct ccs_io_buffer *head)
+void ccs_read_audit_log(struct ccs_io_buffer *head)
 {
-	struct ccs_log_entry *ptr = NULL;
+	struct ccs_audit_log_entry *ptr = NULL;
+	const bool is_granted = head->type == CCS_GRANTLOG;
 	if (head->read_avail)
 		return;
 	if (head->read_buf) {
@@ -492,11 +485,11 @@ void ccs_read_grant_log(struct ccs_io_buffer *head)
 		head->readbuf_size = 0;
 	}
 	spin_lock(&ccs_audit_log_lock);
-	if (!list_empty(&ccs_grant_log)) {
-		ptr = list_entry(ccs_grant_log.next, struct ccs_log_entry,
-				 list);
+	if (!list_empty(&ccs_audit_log[is_granted])) {
+		ptr = list_entry(ccs_audit_log[is_granted].next,
+				 struct ccs_audit_log_entry, list);
 		list_del(&ptr->list);
-		ccs_grant_log_count--;
+		ccs_audit_log_count[is_granted]--;
 		ccs_audit_log_memory_size -= ptr->size;
 	}
 	spin_unlock(&ccs_audit_log_lock);
@@ -509,70 +502,23 @@ void ccs_read_grant_log(struct ccs_io_buffer *head)
 }
 
 /**
- * ccs_poll_grant_log - Wait for a grant log.
+ * ccs_poll_audit_log - Wait for an audit log.
  *
  * @file: Pointer to "struct file".
  * @wait: Pointer to "poll_table".
  *
  * Returns POLLIN | POLLRDNORM when ready to read a grant log.
  */
-int ccs_poll_grant_log(struct file *file, poll_table *wait)
+int ccs_poll_audit_log(struct file *file, poll_table *wait)
 {
-	if (ccs_grant_log_count)
+	struct ccs_io_buffer *head = file->private_data;
+	const bool is_granted = head->type == CCS_GRANTLOG;
+	if (ccs_audit_log_count[is_granted])
 		return POLLIN | POLLRDNORM;
-	poll_wait(file, &ccs_grant_log_wait, wait);
-	if (ccs_grant_log_count)
+	poll_wait(file, &ccs_audit_log_wait[is_granted], wait);
+	if (ccs_audit_log_count[is_granted])
 		return POLLIN | POLLRDNORM;
 	return 0;
 }
 
-/**
- * ccs_read_reject_log - Read a reject log.
- *
- * @head: Pointer to "struct ccs_io_buffer".
- */
-void ccs_read_reject_log(struct ccs_io_buffer *head)
-{
-	struct ccs_log_entry *ptr = NULL;
-	if (head->read_avail)
-		return;
-	if (head->read_buf) {
-		kfree(head->read_buf);
-		head->read_buf = NULL;
-		head->readbuf_size = 0;
-	}
-	spin_lock(&ccs_audit_log_lock);
-	if (!list_empty(&ccs_reject_log)) {
-		ptr = list_entry(ccs_reject_log.next, struct ccs_log_entry,
-				 list);
-		list_del(&ptr->list);
-		ccs_reject_log_count--;
-		ccs_audit_log_memory_size -= ptr->size;
-	}
-	spin_unlock(&ccs_audit_log_lock);
-	if (ptr) {
-		head->read_buf = ptr->log;
-		head->read_avail = strlen(ptr->log) + 1;
-		head->readbuf_size = head->read_avail;
-		kfree(ptr);
-	}
-}
-
-/**
- * ccs_poll_reject_log - Wait for a reject log.
- *
- * @file: Pointer to "struct file".
- * @wait: Pointer to "poll_table".
- *
- * Returns POLLIN | POLLRDNORM when ready to read a reject log.
- */
-int ccs_poll_reject_log(struct file *file, poll_table *wait)
-{
-	if (ccs_reject_log_count)
-		return POLLIN | POLLRDNORM;
-	poll_wait(file, &ccs_reject_log_wait, wait);
-	if (ccs_reject_log_count)
-		return POLLIN | POLLRDNORM;
-	return 0;
-}
 #endif
