@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/compat.h
  *
- * Copyright (C) 2005-2009  NTT DATA CORPORATION
+ * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1   2009/11/11
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -61,10 +61,10 @@
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
 #define mutex semaphore
 #define mutex_init(mutex) init_MUTEX(mutex)
-#define mutex_lock(mutex) down(mutex)
 #define mutex_unlock(mutex) up(mutex)
+#define mutex_lock(mutex) down(mutex)
 #define mutex_lock_interruptible(mutex) down_interruptible(mutex)
-#define mutex_trylock(mutex) !down_trylock(mutex)
+#define mutex_trylock(mutex) (!down_trylock(mutex))
 #define DEFINE_MUTEX(mutexname) DECLARE_MUTEX(mutexname)
 #endif
 
@@ -108,19 +108,22 @@
 	})
 #endif
 
-#ifndef list_for_each_rcu
-#define list_for_each_rcu(pos, head)			\
-	for (pos = rcu_dereference((head)->next);	\
-	     prefetch(pos->next), pos != (head);	\
-	     pos = rcu_dereference(pos->next))
+#ifndef list_for_each_entry_rcu
+#define list_for_each_entry_rcu(pos, head, member)		 \
+	for (pos = list_entry(rcu_dereference((head)->next),	 \
+			      typeof(*pos), member);		 \
+	     prefetch(pos->member.next), &pos->member != (head); \
+	     pos = list_entry(rcu_dereference(pos->member.next), \
+			      typeof(*pos), member))
 #endif
 
-#ifndef list_for_each_entry_rcu
-#define list_for_each_entry_rcu(pos, head, member)			\
-	for (pos = list_entry(rcu_dereference((head)->next), typeof(*pos), \
-			      member);					\
-	     prefetch(pos->member.next), &pos->member != (head);	\
-	     pos = list_entry(rcu_dereference(pos->member.next),	\
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 34)
+#undef list_for_each_entry_rcu
+#define list_for_each_entry_rcu(pos, head, member)			   \
+	for (pos = list_entry(srcu_dereference((head)->next, &ccs_ss),	   \
+			      typeof(*pos), member);			   \
+	     prefetch(pos->member.next), &pos->member != (head);	   \
+	     pos = list_entry(srcu_dereference(pos->member.next, &ccs_ss), \
 			      typeof(*pos), member))
 #endif
 
@@ -203,12 +206,12 @@ static inline struct socket *SOCKET_I(struct inode *inode)
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
-
+#ifndef _STRUCT_PATH_DEFINED
 struct path {
 	struct vfsmount *mnt;
 	struct dentry *dentry;
 };
-
+#endif
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
