@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1+   2010/02/14
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -26,19 +26,28 @@
 #include <asm/uaccess.h>
 #include <stdarg.h>
 #include <linux/delay.h>
+#include <linux/sched.h>
 #include <linux/version.h>
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-#include <linux/kmod.h>
-#endif
 #include <linux/in6.h>
 #include <linux/ccsecurity.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
-#include <linux/err.h>
-#else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
 #include <linux/fs.h>
 #endif
 #include "compat.h"
 
+/* Prototype definition for "struct ccsecurity_operations". */
+
+void __init ccs_policy_io_init(void);
+void __init ccs_mm_init(void);
+void __init ccs_domain_init(void);
+void __init ccs_file_init(void);
+void __init ccs_network_init(void);
+void __init ccs_signal_init(void);
+void __init ccs_capability_init(void);
+void __init ccs_mount_init(void);
+void __init ccs_maymount_init(void);
+void __init ccs_autobind_init(void);
+
 /* Index numbers for Access Controls. */
 enum ccs_acl_entry_type_index {
 	CCS_TYPE_PATH_ACL,
@@ -75,6 +84,7 @@ enum ccs_path_acl_index {
 	CCS_TYPE_REWRITE,
 	CCS_TYPE_CHROOT,
 	CCS_TYPE_UMOUNT,
+	CCS_TYPE_TRANSIT,
 	CCS_MAX_PATH_OPERATION
 };
 
@@ -162,6 +172,7 @@ enum ccs_mac_index {
 	CCS_MAC_FILE_MOUNT,
 	CCS_MAC_FILE_UMOUNT,
 	CCS_MAC_FILE_PIVOT_ROOT,
+	CCS_MAC_FILE_TRANSIT,
 	CCS_MAC_NETWORK_UDP_BIND,
 	CCS_MAC_NETWORK_UDP_CONNECT,
 	CCS_MAC_NETWORK_TCP_BIND,
@@ -294,23 +305,27 @@ enum ccs_conditions_index {
 #define ROOT_NAME_LEN                     (sizeof(ROOT_NAME) - 1)
 
 /* Value type definition. */
-#define CCS_VALUE_TYPE_INVALID     0
-#define CCS_VALUE_TYPE_DECIMAL     1
-#define CCS_VALUE_TYPE_OCTAL       2
-#define CCS_VALUE_TYPE_HEXADECIMAL 3
+enum ccs_value_type {
+	CCS_VALUE_TYPE_INVALID,
+	CCS_VALUE_TYPE_DECIMAL,
+	CCS_VALUE_TYPE_OCTAL,
+	CCS_VALUE_TYPE_HEXADECIMAL
+};
 
 #define CCS_EXEC_TMPSIZE     4096
 
 /* Profile number is an integer between 0 and 255. */
 #define CCS_MAX_PROFILES 256
 
-#define CCS_CONFIG_DISABLED          0
-#define CCS_CONFIG_LEARNING          1
-#define CCS_CONFIG_PERMISSIVE        2
-#define CCS_CONFIG_ENFORCING         3
-#define CCS_CONFIG_WANT_REJECT_LOG  64
-#define CCS_CONFIG_WANT_GRANT_LOG  128
-#define CCS_CONFIG_USE_DEFAULT     255
+enum ccs_mode_value {
+	CCS_CONFIG_DISABLED,
+	CCS_CONFIG_LEARNING,
+	CCS_CONFIG_PERMISSIVE,
+	CCS_CONFIG_ENFORCING,
+	CCS_CONFIG_WANT_REJECT_LOG =  64,
+	CCS_CONFIG_WANT_GRANT_LOG  = 128,
+	CCS_CONFIG_USE_DEFAULT     = 255
+};
 
 #define CCS_OPEN_FOR_READ_TRUNCATE        4
 #define CCS_OPEN_FOR_IOCTL_ONLY           8
@@ -323,8 +338,6 @@ enum ccs_conditions_index {
 struct dentry;
 struct vfsmount;
 struct in6_addr;
-extern asmlinkage long sys_getpid(void);
-extern asmlinkage long sys_getppid(void);
 
 /**
  * list_for_each_cookie - iterate over a list with cookie.
@@ -337,12 +350,21 @@ extern asmlinkage long sys_getppid(void);
  * @cookie must be NULL when iteration starts, and @cookie will become
  * NULL when iteration finishes.
  */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 34)
 #define list_for_each_cookie(pos, cookie, head)				\
 	for (({ if (!cookie)						\
 				     cookie = head; }),			\
 		     pos = rcu_dereference((cookie)->next);		\
 	     prefetch(pos->next), pos != (head) || ((cookie) = NULL);	\
 	     (cookie) = pos, pos = rcu_dereference(pos->next))
+#else
+#define list_for_each_cookie(pos, cookie, head)				\
+	for (({ if (!cookie)						\
+				     cookie = head; }),			\
+		     pos = srcu_dereference((cookie)->next, &ccs_ss);	\
+	     prefetch(pos->next), pos != (head) || ((cookie) = NULL);	\
+	     (cookie) = pos, pos = srcu_dereference(pos->next, &ccs_ss))
+#endif
 
 struct ccs_name_union {
 	const struct ccs_path_info *filename;
@@ -481,6 +503,8 @@ struct ccs_condition {
 
 struct ccs_execve_entry;
 
+#define CCS_RETRY_REQUEST 1 /* Retry this request. */
+
 /* Structure for request info. */
 struct ccs_request_info {
 	struct ccs_domain_info *domain;
@@ -512,7 +536,7 @@ struct ccs_execve_entry {
 	int reader_idx;
 	/* For execute_handler */
 	const struct ccs_path_info *handler;
-	char *handler_path; /* = kstrdup(handler->name, GFP_KERNEL) */
+	char *handler_path; /* = kstrdup(handler->name, CCS_GFP_FLAGS) */
 	/* For dumping argv[] and envp[]. */
 	struct ccs_page_dump dump;
 	/* For temporary use. */
@@ -785,7 +809,7 @@ struct ccs_io_buffer {
 	int (*poll) (struct file *file, poll_table *wait);
 	/* Exclusive lock for this structure.   */
 	struct mutex io_sem;
-	/* Index returned by ccs_read_lock().   */
+	/* Index returned by ccs_lock().        */
 	int reader_idx;
 	/* The position currently reading from. */
 	struct list_head *read_var1;
@@ -848,11 +872,10 @@ struct ccs_profile {
 		  + CCS_MAX_MAC_CATEGORY_INDEX];
 };
 
-/* Prototype definition. */
+/* Prototype definition for internal use. */
 
 bool ccs_address_matches_group(const bool is_ipv6, const u32 *address,
 			       const struct ccs_address_group *group);
-bool ccs_commit_ok(void *ptr, void *data, const unsigned int size);
 bool ccs_compare_name_union(const struct ccs_path_info *name,
 			    const struct ccs_name_union *ptr);
 bool ccs_compare_number_union(const unsigned long value,
@@ -916,13 +939,16 @@ int ccs_get_mode(const u8 profile, const u8 index);
 int ccs_get_path(const char *pathname, struct path *path);
 int ccs_init_request_info(struct ccs_request_info *r,
 			  struct ccs_domain_info *domain, const u8 index);
+int ccs_lock(void);
+int ccs_may_transit(const char *domainname, const char *pathname);
 int ccs_open_control(const u8 type, struct file *file);
 int ccs_parse_ip_address(char *address, u16 *min, u16 *max);
+int ccs_path_permission(struct ccs_request_info *r, u8 operation,
+			const struct ccs_path_info *filename);
 int ccs_poll_control(struct file *file, poll_table *wait);
 int ccs_poll_audit_log(struct file *file, poll_table *wait);
 int ccs_read_control(struct file *file, char __user *buffer,
 		     const int buffer_len);
-int ccs_read_lock(void);
 int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
      __attribute__ ((format(printf, 2, 3)));
 int ccs_symlink_path(const char *pathname, struct ccs_path_info *name);
@@ -973,10 +999,8 @@ struct ccs_number_group *ccs_get_number_group(const char *group_name);
 struct ccs_path_group *ccs_get_path_group(const char *group_name);
 struct ccs_profile *ccs_profile(const u8 profile);
 u8 ccs_parse_ulong(unsigned long *result, char **str);
-void ccs_check_profile(void);
 void ccs_fill_path_info(struct ccs_path_info *ptr);
 void ccs_get_attributes(struct ccs_obj_info *obj);
-void ccs_load_policy(const char *filename);
 void ccs_memory_free(const void *ptr, size_t size);
 void ccs_normalize_line(unsigned char *buffer);
 void ccs_print_ipv6(char *buffer, const int buffer_len,
@@ -987,11 +1011,12 @@ void ccs_put_name_union(struct ccs_name_union *ptr);
 void ccs_put_number_union(struct ccs_number_union *ptr);
 void ccs_read_audit_log(struct ccs_io_buffer *head);
 void ccs_read_memory_counter(struct ccs_io_buffer *head);
-void ccs_read_unlock(const int idx);
 void ccs_run_gc(void);
+void ccs_unlock(const int idx);
 void ccs_warn_log(struct ccs_request_info *r, const char *fmt, ...)
      __attribute__ ((format(printf, 2, 3)));
 void ccs_warn_oom(const char *function);
+void *ccs_commit_ok(void *data, const unsigned int size);
 
 /* strcmp() for "struct ccs_path_info" structure. */
 static inline bool ccs_pathcmp(const struct ccs_path_info *a,
@@ -1000,11 +1025,131 @@ static inline bool ccs_pathcmp(const struct ccs_path_info *a,
 	return a->hash != b->hash || strcmp(a->name, b->name);
 }
 
-static inline int ccs_memcmp(void *a, void *b, const size_t offset,
-			     const size_t size)
+static inline bool ccs_is_same_acl_head(const struct ccs_acl_info *p1,
+					const struct ccs_acl_info *p2)
 {
-	return memcmp(((char *) a) + offset, ((char *) b) + offset,
-		      size - offset);
+	return p1->type == p2->type && p1->cond == p2->cond;
+}
+
+static inline bool ccs_is_same_name_union(const struct ccs_name_union *p1,
+					  const struct ccs_name_union *p2)
+{
+	return p1->filename == p2->filename && p1->group == p2->group &&
+		p1->is_group == p2->is_group;
+}
+
+static inline bool ccs_is_same_number_union(const struct ccs_number_union *p1,
+					    const struct ccs_number_union *p2)
+{
+	return p1->values[0] == p2->values[0] && p1->values[1] == p2->values[1]
+		&& p1->group == p2->group && p1->min_type == p2->min_type &&
+		p1->max_type == p2->max_type && p1->is_group == p2->is_group;
+}
+
+static inline bool ccs_is_same_path_acl(const struct ccs_path_acl *p1,
+					const struct ccs_path_acl *p2)
+{
+	return ccs_is_same_acl_head(&p1->head, &p2->head) &&
+		ccs_is_same_name_union(&p1->name, &p2->name);
+}
+
+static inline bool ccs_is_same_path_number3_acl
+(const struct ccs_path_number3_acl *p1,
+ const struct ccs_path_number3_acl *p2)
+{
+	return ccs_is_same_acl_head(&p1->head, &p2->head)
+		&& ccs_is_same_name_union(&p1->name, &p2->name)
+		&& ccs_is_same_number_union(&p1->mode, &p2->mode)
+		&& ccs_is_same_number_union(&p1->major, &p2->major)
+		&& ccs_is_same_number_union(&p1->minor, &p2->minor);
+}
+
+static inline bool ccs_is_same_path2_acl(const struct ccs_path2_acl *p1,
+					 const struct ccs_path2_acl *p2)
+{
+	return ccs_is_same_acl_head(&p1->head, &p2->head)
+		&& ccs_is_same_name_union(&p1->name1, &p2->name1)
+		&& ccs_is_same_name_union(&p1->name2, &p2->name2);
+}
+
+static inline bool ccs_is_same_path_number_acl
+(const struct ccs_path_number_acl *p1, const struct ccs_path_number_acl *p2)
+{
+	return ccs_is_same_acl_head(&p1->head, &p2->head)
+		&& ccs_is_same_name_union(&p1->name, &p2->name)
+		&& ccs_is_same_number_union(&p1->number, &p2->number);
+}
+
+static inline bool ccs_is_same_mount_acl(const struct ccs_mount_acl *p1,
+					 const struct ccs_mount_acl *p2)
+{
+	return ccs_is_same_acl_head(&p1->head, &p2->head) &&
+		ccs_is_same_name_union(&p1->dev_name, &p2->dev_name) &&
+		ccs_is_same_name_union(&p1->dir_name, &p2->dir_name) &&
+		ccs_is_same_name_union(&p1->fs_type, &p2->fs_type) &&
+		ccs_is_same_number_union(&p1->flags, &p2->flags);
+}
+
+static inline bool ccs_is_same_ip_network_acl
+(const struct ccs_ip_network_acl *p1, const struct ccs_ip_network_acl *p2)
+{
+	return ccs_is_same_acl_head(&p1->head, &p2->head)
+		&& p1->address_type == p2->address_type &&
+		p1->address.ipv4.min == p2->address.ipv4.min &&
+		p1->address.ipv6.min == p2->address.ipv6.min &&
+		p1->address.ipv4.max == p2->address.ipv4.max &&
+		p1->address.ipv6.max == p2->address.ipv6.max &&
+		p1->address.group == p2->address.group &&
+		ccs_is_same_number_union(&p1->port, &p2->port);
+}
+
+static inline bool ccs_is_same_address_group_member
+(const struct ccs_address_group_member *p1,
+ const struct ccs_address_group_member *p2)
+{
+	return p1->is_ipv6 == p2->is_ipv6 &&
+		p1->min.ipv4 == p2->min.ipv4 && p1->min.ipv6 == p2->min.ipv6 &&
+		p1->max.ipv4 == p2->max.ipv4 && p1->max.ipv6 == p2->max.ipv6;
+}
+
+static inline bool ccs_is_same_domain_initializer_entry
+(const struct ccs_domain_initializer_entry *p1,
+ const struct ccs_domain_initializer_entry *p2)
+{
+	return p1->is_not == p2->is_not && p1->is_last_name == p2->is_last_name
+		&& p1->domainname == p2->domainname
+		&& p1->program == p2->program;
+}
+
+static inline bool ccs_is_same_domain_keeper_entry
+(const struct ccs_domain_keeper_entry *p1,
+ const struct ccs_domain_keeper_entry *p2)
+{
+	return p1->is_not == p2->is_not && p1->is_last_name == p2->is_last_name
+		&& p1->domainname == p2->domainname
+		&& p1->program == p2->program;
+}
+
+static inline bool ccs_is_same_aggregator_entry
+(const struct ccs_aggregator_entry *p1,
+ const struct ccs_aggregator_entry *p2)
+{
+	return p1->original_name == p2->original_name &&
+		p1->aggregated_name == p2->aggregated_name;
+}
+
+static inline bool ccs_is_same_condition(const struct ccs_condition *p1,
+					 const struct ccs_condition *p2)
+{
+	return p1->size == p2->size && p1->condc == p2->condc &&
+		p1->numbers_count == p2->numbers_count &&
+		p1->names_count == p2->names_count &&
+		p1->argc == p2->argc && p1->envc == p2->envc &&
+		p1->post_state[0] == p2->post_state[0] &&
+		p1->post_state[1] == p2->post_state[1] &&
+		p1->post_state[2] == p2->post_state[2] &&
+		p1->post_state[3] == p2->post_state[3] &&
+		!memcmp(p1 + 1, p2 + 1, p1->size - sizeof(*p1));
 }
 
 #define CCS_HASH_BITS 8
@@ -1026,11 +1171,31 @@ extern struct list_head ccs_reservedport_list;
 extern struct list_head ccs_policy_manager_list;
 extern struct list_head ccs_address_list;
 extern struct list_head ccs_condition_list;
-extern struct mutex ccs_name_list_lock;
 extern struct list_head ccs_name_list[CCS_MAX_HASH];
-
 extern bool ccs_policy_loaded;
 extern struct ccs_domain_info ccs_kernel_domain;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
+extern struct srcu_struct ccs_ss;
+
+static inline int ccs_read_lock(void)
+{
+	return srcu_read_lock(&ccs_ss);
+}
+
+static inline void ccs_read_unlock(const int idx)
+{
+	srcu_read_unlock(&ccs_ss, idx);
+}
+#else
+static inline int ccs_read_lock(void)
+{
+	return ccs_lock();
+}
+static inline void ccs_read_unlock(const int idx)
+{
+	ccs_unlock(idx);
+}
+#endif
 
 extern const char *ccs_condition_keyword[CCS_MAX_CONDITION_KEYWORD];
 
@@ -1040,19 +1205,18 @@ extern unsigned int ccs_query_memory_size;
 extern unsigned int ccs_quota_for_query;
 
 #include <linux/dcache.h>
-extern spinlock_t vfsmount_lock;
 
 #ifdef D_PATH_DISCONNECT
 
 static inline void ccs_realpath_lock(void)
 {
-	spin_lock(&vfsmount_lock);
+	spin_lock(ccsecurity_exports.vfsmount_lock);
 	spin_lock(&dcache_lock);
 }
 static inline void ccs_realpath_unlock(void)
 {
 	spin_unlock(&dcache_lock);
-	spin_unlock(&vfsmount_lock);
+	spin_unlock(ccsecurity_exports.vfsmount_lock);
 }
 
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
@@ -1060,11 +1224,11 @@ static inline void ccs_realpath_unlock(void)
 static inline void ccs_realpath_lock(void)
 {
 	spin_lock(&dcache_lock);
-	spin_lock(&vfsmount_lock);
+	spin_lock(ccsecurity_exports.vfsmount_lock);
 }
 static inline void ccs_realpath_unlock(void)
 {
-	spin_unlock(&vfsmount_lock);
+	spin_unlock(ccsecurity_exports.vfsmount_lock);
 	spin_unlock(&dcache_lock);
 }
 
@@ -1190,4 +1354,20 @@ static inline void ccs_put_name(const struct ccs_path_info *name)
 	}
 }
 
+#ifndef __GFP_HIGHIO
+#define __GFP_HIGHIO 0
+#endif
+#ifndef __GFP_NOWARN
+#define __GFP_NOWARN 0
+#endif
+#ifndef __GFP_NORETRY
+#define __GFP_NORETRY 0
+#endif
+#ifndef __GFP_NOMEMALLOC
+#define __GFP_NOMEMALLOC 0
+#endif
+
+#define CCS_GFP_FLAGS (__GFP_WAIT | __GFP_IO | __GFP_HIGHIO | __GFP_NOWARN | \
+		       __GFP_NORETRY | __GFP_NOMEMALLOC)
+
 #endif
