@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2010  NTT DATA CORPORATION
  *
- * Version: 1.7.1+   2010/03/26
+ * Version: 1.7.2   2010/04/01
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -54,22 +54,26 @@ bool ccs_memory_ok(const void *ptr, const unsigned int size)
 }
 
 /**
- * ccs_commit_ok - Check memory quota.
+ * ccs_commit_ok - Allocate memory and check memory quota.
  *
- * @ptr:    Pointer to allocated memory.
  * @data:   Data to copy from.
  * @size:   Size in byte.
  *
- * Returns true if @ptr is not NULL and quota not exceeded, false otherwise.
+ * Returns pointer to allocated memory on success, NULL otherwise.
+ * @data is zero-cleared on success.
  */
-bool ccs_commit_ok(void *ptr, void *data, const unsigned int size)
+void *ccs_commit_ok(void *data, const unsigned int size)
 {
+	void *ptr = kmalloc(size, CCS_GFP_FLAGS);
+	if (!ptr)
+		return NULL;
 	if (ccs_memory_ok(ptr, size)) {
 		memmove(ptr, data, size);
 		memset(data, 0, size);
-		return true;
+		return ptr;
 	}
-	return false;
+	kfree(ptr);
+	return NULL;
 }
 
 
@@ -99,12 +103,13 @@ LIST_HEAD(ccs_address_list);
 const struct in6_addr *ccs_get_ipv6_address(const struct in6_addr *addr)
 {
 	struct ccs_ipv6addr_entry *entry;
-	struct ccs_ipv6addr_entry *ptr;
+	struct ccs_ipv6addr_entry *ptr = NULL;
 	int error = -ENOMEM;
 	if (!addr)
 		return NULL;
-	entry = kzalloc(sizeof(*entry), GFP_KERNEL);
-	mutex_lock(&ccs_policy_lock);
+	entry = kzalloc(sizeof(*entry), CCS_GFP_FLAGS);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		goto out;
 	list_for_each_entry(ptr, &ccs_address_list, list) {
 		if (memcmp(&ptr->addr, addr, sizeof(*addr)))
 			continue;
@@ -121,13 +126,13 @@ const struct in6_addr *ccs_get_ipv6_address(const struct in6_addr *addr)
 		error = 0;
 	}
 	mutex_unlock(&ccs_policy_lock);
+ out:
 	kfree(entry);
 	return !error ? &ptr->addr : NULL;
 }
 
 /* The list for "struct ccs_name_entry". */
 struct list_head ccs_name_list[CCS_MAX_HASH];
-DEFINE_MUTEX(ccs_name_list_lock);
 
 /**
  * ccs_get_name - Allocate memory for string data.
@@ -153,7 +158,8 @@ const struct ccs_path_info *ccs_get_name(const char *name)
 #else
 	head = &ccs_name_list[hash % CCS_MAX_HASH];
 #endif
-	mutex_lock(&ccs_name_list_lock);
+	if (mutex_lock_interruptible(&ccs_policy_lock))
+		return NULL;
 	list_for_each_entry(ptr, head, list) {
 		if (hash != ptr->entry.hash || strcmp(name, ptr->entry.name))
 			continue;
@@ -161,7 +167,7 @@ const struct ccs_path_info *ccs_get_name(const char *name)
 		goto out;
 	}
 	allocated_len = ccs_round2(sizeof(*ptr) + len);
-	ptr = kzalloc(allocated_len, GFP_KERNEL);
+	ptr = kzalloc(allocated_len, CCS_GFP_FLAGS);
 	if (!ptr || (ccs_quota_for_policy &&
 		     atomic_read(&ccs_policy_memory_size) + allocated_len
 		     > ccs_quota_for_policy)) {
@@ -178,23 +184,22 @@ const struct ccs_path_info *ccs_get_name(const char *name)
 	ptr->size = allocated_len;
 	list_add_tail(&ptr->list, head);
  out:
-	mutex_unlock(&ccs_name_list_lock);
+	mutex_unlock(&ccs_policy_lock);
 	return ptr ? &ptr->entry : NULL;
 }
 
 /**
  * ccs_mm_init - Initialize mm related code.
- *
- * Returns 0.
  */
-static int __init ccs_mm_init(void)
+void __init ccs_mm_init(void)
 {
-	int i;
-	for (i = 0; i < CCS_MAX_HASH; i++)
-		INIT_LIST_HEAD(&ccs_name_list[i]);
+	int idx;
+	for (idx = 0; idx < CCS_MAX_HASH; idx++)
+		INIT_LIST_HEAD(&ccs_name_list[idx]);
 	INIT_LIST_HEAD(&ccs_kernel_domain.acl_info_list);
 	ccs_kernel_domain.domainname = ccs_get_name(ROOT_NAME);
 	list_add_tail_rcu(&ccs_kernel_domain.list, &ccs_domain_list);
+	idx = ccs_read_lock();
 	if (ccs_find_domain(ROOT_NAME) != &ccs_kernel_domain)
 		panic("Can't register ccs_kernel_domain");
 	{
@@ -211,15 +216,9 @@ static int __init ccs_mm_init(void)
 			cp = cp2;
 		}
 	}
-	return 0;
+	ccs_read_unlock(idx);
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-__initcall(ccs_mm_init);
-#else
-core_initcall(ccs_mm_init);
-#endif
-
 unsigned int ccs_audit_log_memory_size;
 unsigned int ccs_quota_for_audit_log;
 
